/**
 * P4
 * http://snarfed.org/space/p4
 *
 * Copyright 2001-2004
 * Ken Ashcraft <kashcraft+p4@gmail.com>
 * Ryan Barrett <p4@ryanb.org>
 * Maulik Shah <p4@maulik.net>
 * Nathan Stoll <nathanstoll+p4@gmail.com>
 *
 * @file msg_hello.h
 *
 * Defines the cmsg_hello class, which is derived from the abstract cmsg class.
 * The cmsg_hello class is the class that HELLO messages instantiate. For more
 * information, see protocol.txt, msg.h, and protocol.h.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef _MSG_HELLO_H
#define _MSG_HELLO_H

#include "msg.h"
#include "protocol.h"


/*****************************************************************************
 * FORWARD DECLARATIONS
 *****************************************************************************/
class cmsg_factory;  // so that we can declare it as a friend


/*****************************************************************************
 * CMSG_HELLO CLASS
 *****************************************************************************/

class cmsg_hello : public cmsg {
  public:

  /** ctor
   */
  cmsg_hello();

  /** dtor
   */
  virtual ~cmsg_hello() {}

  /** broadcast
   *
   * HELLO messages are broadcast, so the broadcast method is made public.
   * We use send() in the bootstrap phase.
   */
  virtual uint broadcast() { return cmsg::broadcast(); }
  virtual const cconn &send(const string &username);


  protected:
  friend class cmsg_factory;  // for constructing from a p4 header

  /** ctors
   */
  cmsg_hello(const p4_header &hdr);
  cmsg_hello(const cmsg &msg);

  /** virt_process
   *
   * For a HELLO message, virt_process parses the message data and adds an
   * entry to the LKA. It then forwards the HELLO to all connections *except*
   * the connection it was received on.
   */
  virtual bool virt_process() const;

  /** debug_print
   *
   * Prints the hello data, including destination, # plugins, and plugin ids.
   */
  void debug_print() const;


  private:
  // private since it's a helper. to use, call ntoh and hton in cmsg, which
  // are protected.
  virtual void convert_byteorder(short_fn sfn, long_fn lfn);

  /**************************************************************************
   * MEMBERS
   **************************************************************************/
  p4_hello  *hello;
  plugin_info *plugins;
};


#endif  // _MSG_HELLO_H
