/**
 * P4
 * http://snarfed.org/space/p4
 *
 * Copyright 2001-2004
 * Ken Ashcraft <kashcraft+p4@gmail.com>
 * Ryan Barrett <p4@ryanb.org>
 * Maulik Shah <p4@maulik.net>
 * Nathan Stoll <nathanstoll+p4@gmail.com>
 *
 * @file msg_lka_ack.h
 *
 * Defines the cmsg_lka_ack class, which is derived from the abstract cmsg
 * class. The cmsg_lka_ack class is the class that LKA_ACK messages
 * instantiate. For more information, see protocol.txt, msg.h, and protocol.h.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef _MSG_LKA_ACK_H
#define _MSG_LKA_ACK_H

#include "protocol.h"
#include "msg.h"
#include "lka.h"

#include <pthread.h>


/*****************************************************************************
 * FORWARD DECLARATIONS
 *****************************************************************************/
class cmsg_factory;  // so that we can declare it as a friend


/*****************************************************************************
 * CMSG_LKA_ACK CLASS
 *****************************************************************************/

class cmsg_lka_ack : public cmsg {
  public:

  /** ctor
   */
  cmsg_lka_ack();

  /** dtor
   */
  virtual ~cmsg_lka_ack() { }// delete data; } 

  /** send
   *
   * LKA_ACK messages are sent direct, so the send methods are made public.
   */
  virtual const cconn &send(const string &username)
    { return cmsg::send(username); }

  /** synchronization members
   */
  static pthread_mutex_t received_mtx;
  static pthread_cond_t received_cond;
  static bool received;

  protected:
  friend class cmsg_factory;  // for constructing from a p4 header

  /** ctors
   */
  cmsg_lka_ack(const p4_header &hdr) : cmsg(hdr) {}
  cmsg_lka_ack(const cmsg &msg) : cmsg(msg) {}

  /** virt_process
   *
   * For a LKA_ACK message, virt_process adds the entries to the LKA.
   */
  virtual bool virt_process() const;

  private:
  // private since it's a helper. to use, call ntoh and hton in cmsg, which
  // are protected.
  virtual void convert_byteorder(short_fn sfn, long_fn lfn);

  /**************************************************************************
   * MEMBERS
   **************************************************************************/
  // no new members, just remember that data always points to a p4_lka_ack 
};


#endif  // _MSG_LKA_ACK_H
